@@ -1,4 +1,6 @@
-import copy, numpy, pandas, pyarrow as pa, sys, uuid
+import copy, hashlib, logging, numpy, pandas as pd, pyarrow as pa, sys, uuid
+from functools import lru_cache
+from weakref import WeakValueDictionary
 
 from .util import (error, in_ipython, make_iframe, random_string, warn)
 
@@ -21,6 +23,30 @@ try:
 except ImportError:
     1
 
+logger = logging.getLogger('Plotter')
+
+CACHE_COERCION_SIZE = 100
+
+
+_cache_coercion_val = None
+@lru_cache(maxsize=CACHE_COERCION_SIZE)
+def cache_coercion_helper(k):
+    return _cache_coercion_val
+
+def cache_coercion(k, v):
+    """
+        Holds references to last 100 used coercions
+        Use with weak key/value dictionaries for actual lookups
+    """
+    global _cache_coercion_val
+    _cache_coercion_val = v
+
+    return cache_coercion_helper(k)
+
+class WeakValueWrapper:
+    def __init__(self, v):
+        self.v = v
+
 class Plotter(object):
     """Graph plotting class.
 
@@ -30,12 +56,14 @@ class Plotter(object):
 
     To streamline reuse and replayable notebooks, Plotter manipulations are immutable. Each chained call returns a new instance that derives from the previous one. The old plotter or the new one can then be used to create different graphs.
 
+    When using memoization, for .register(api=3) sessions with .plot(memoize=True), Pandas/cudf arrow coercions are memoized, and file uploads are skipped on same-hash dataframes.
+
     The class supports convenience methods for mixing calls across Pandas, NetworkX, and IGraph.
     """
 
-
     _defaultNodeId = '__nodeid__'
-
+    _pd_hash_to_arrow = WeakValueDictionary()
+    _cudf_hash_to_arrow = WeakValueDictionary()
 
     def __init__(self):
         # Bindings
@@ -295,7 +323,7 @@ class Plotter(object):
         **Example: See encode_point_color**
         """
 
-        return self.__encode('edge', 'color',  'edgeColorEncoding',
+        return self.__encode('edge', 'color', 'edgeColorEncoding',
             column=column, palette=palette, as_categorical=as_categorical, as_continuous=as_continuous,
             categorical_mapping=categorical_mapping, default_mapping=default_mapping,
             for_default=for_default, for_current=for_current)
@@ -333,7 +361,7 @@ class Plotter(object):
                 g2b = g.encode_point_size('brands', categorical_mapping={'toyota': 100, 'ford': 200}, default_mapping=50)
 
         """
-        return self.__encode('point', 'size',  'pointSizeEncoding', column=column,
+        return self.__encode('point', 'size', 'pointSizeEncoding', column=column,
             categorical_mapping=categorical_mapping, default_mapping=default_mapping,
             for_default=for_default, for_current=for_current)
 
@@ -397,7 +425,7 @@ class Plotter(object):
 
         """
 
-        return self.__encode('point', 'icon',  'pointIconEncoding', column=column,
+        return self.__encode('point', 'icon', 'pointIconEncoding', column=column,
             categorical_mapping=categorical_mapping, continuous_binning=continuous_binning, default_mapping=default_mapping,
             comparator=comparator,
             for_default=for_default, for_current=for_current,
@@ -452,7 +480,7 @@ class Plotter(object):
                 g2b = g.encode_edge_icon('country', border={'width': 3, color: 'black', 'stroke': 'dashed'}, 'categorical_mapping={'England': 'UK', 'America': 'US'})
 
         """
-        return self.__encode('edge', 'icon',   'edgeIconEncoding', column=column,
+        return self.__encode('edge', 'icon', 'edgeIconEncoding', column=column,
             categorical_mapping=categorical_mapping, continuous_binning=continuous_binning, default_mapping=default_mapping,
             comparator=comparator,
             for_default=for_default, for_current=for_current,
@@ -498,12 +526,12 @@ class Plotter(object):
             default_mapping=default_mapping,
             for_current=for_current, for_default=for_default,
             as_text=as_text, blend_mode=blend_mode, style=style, border=border,
-            continuous_binning=continuous_binning, ##new
-            comparator=comparator, ##new
+            continuous_binning=continuous_binning,
+            comparator=comparator,
             color=color, bg=bg, fg=fg, shape=shape)
 
 
-    def __encode(self, graph_type, feature, feature_binding,
+    def __encode(self, graph_type, feature, feature_binding,  # noqa: C901
             column,
             palette=None,
             as_categorical=None, as_continuous=None,
@@ -522,8 +550,8 @@ class Plotter(object):
 
         if not (graph_type in ['point', 'edge']):
             raise ValueError({
-                    'message': 'graph_type must be "point" or "edge"',
-                    'data': {'graph_type': graph_type } })
+                'message': 'graph_type must be "point" or "edge"',
+                'data': {'graph_type': graph_type } })
 
         if (categorical_mapping is None) and (palette is None) and (continuous_binning is None) and not feature.startswith('badge'):
             return self.bind(**{f'{graph_type}_{feature}': column})
@@ -572,7 +600,7 @@ class Plotter(object):
 
             if as_categorical:
                 raise ValueError({'message': 'as_categorical cannot be True when continuous_binning is provided'})
-            if as_continuous == False:
+            if as_continuous is False:
                 raise ValueError({'message': 'as_continuous cannot be False when continuous_binning is set'})
 
             transform = {
@@ -595,15 +623,14 @@ class Plotter(object):
             'encodingType': feature,
             'attribute': column,
             **transform,
-            **({'bg':        bg} if not         (bg is None) else {}),
-            **({'color':     color} if not      (color is None) else {}),
-            **({'fg':        fg} if not         (fg is None) else {}),
-
-            **({'asText':    as_text} if not    (as_text is None) else {}),
-            **({'blendMode': blend_mode} if not (blend_mode is None) else {}),
-            **({'style':     style} if not      (style is None) else {}),
-            **({'border':    border} if not     (border is None) else {}),
-            **({'shape':     shape} if not      (shape is None) else {})
+            **({'bg':        bg} if not         (bg is None) else {}),  # noqa: E241,E271
+            **({'color':     color} if not      (color is None) else {}),  # noqa: E241,E271
+            **({'fg':        fg} if not         (fg is None) else {}),  # noqa: E241,E271
+            **({'asText':    as_text} if not    (as_text is None) else {}),  # noqa: E241,E271
+            **({'blendMode': blend_mode} if not (blend_mode is None) else {}),  # noqa: E241,E271
+            **({'style':     style} if not      (style is None) else {}),  # noqa: E241,E271
+            **({'border':    border} if not     (border is None) else {}),  # noqa: E241,E271
+            **({'shape':     shape} if not      (shape is None) else {})  # noqa: E241,E271
         }
 
         complex_encodings = copy.deepcopy(self._complex_encodings)
@@ -791,7 +818,7 @@ class Plotter(object):
                 g.plot()
         """
 
-        base = self.bind(node=node) if not node is None else self
+        base = self.bind(node=node) if node is not None else self
         res = copy.copy(base)
         res._nodes = nodes
         return res
@@ -896,7 +923,7 @@ class Plotter(object):
         return res
 
 
-    def plot(self, graph=None, nodes=None, name=None, description=None, render=None, skip_upload=False):
+    def plot(self, graph=None, nodes=None, name=None, description=None, render=None, skip_upload=False, as_files=False, memoize=True):  # noqa: C901
         """Upload data to the Graphistry server and show as an iframe of it.
 
         Uses the currently bound schema structure and visual encodings.
@@ -920,7 +947,13 @@ class Plotter(object):
         :type render: Boolean
 
         :param skip_upload: Return node/edge/bindings that would have been uploaded. By default, upload happens.
-        :type skip_upload: Boolean. 
+        :type skip_upload: Boolean.
+
+        :param as_files: Upload distinct node/edge files under the managed Files PI. Default off, will switch to default-on when stable.
+        :type as_files: Boolean.
+
+        :param memoize: Tries to memoize pandas/cudf->arrow conversion, including skipping upload. Default on.
+        :type memoize: Boolean.
 
         **Example: Simple**
             ::
@@ -958,23 +991,22 @@ class Plotter(object):
         from .pygraphistry import PyGraphistry
         api_version = PyGraphistry.api_version()
         if api_version == 1:
-            dataset = self._plot_dispatch(g, n, name, description, 'json', self._style)
+            dataset = self._plot_dispatch(g, n, name, description, 'json', self._style, memoize)
             if skip_upload:
                 return dataset
             info = PyGraphistry._etl1(dataset)
         elif api_version == 2:
-            dataset = self._plot_dispatch(g, n, name, description, 'vgraph', self._style)
+            dataset = self._plot_dispatch(g, n, name, description, 'vgraph', self._style, memoize)
             if skip_upload:
                 return dataset
             info = PyGraphistry._etl2(dataset)
         elif api_version == 3:
             PyGraphistry.refresh()
-            dataset = self._plot_dispatch(g, n, name, description, 'arrow', self._style)
+            dataset = self._plot_dispatch(g, n, name, description, 'arrow', self._style, memoize)
             if skip_upload:
                 return dataset
-            #fresh
             dataset.token = PyGraphistry.api_token()
-            dataset.post()
+            dataset.post(as_files=as_files, memoize=memoize)
             info = {
                 'name': dataset.dataset_id,
                 'type': 'arrow',
@@ -985,9 +1017,9 @@ class Plotter(object):
         cfg_client_protocol_hostname = PyGraphistry._config['client_protocol_hostname']
         full_url = ('%s:%s' % (PyGraphistry._config['protocol'], viz_url)) if cfg_client_protocol_hostname is None else viz_url
 
-        if (render == False) or ((render is None) and not self._render):
+        if (render is False) or ((render is None) and not self._render):
             return full_url
-        elif (render == True) or in_ipython():
+        elif (render is True) or in_ipython():
             from IPython.core.display import HTML
             return HTML(make_iframe(full_url, self._height))
         else:
@@ -1053,8 +1085,9 @@ class Plotter(object):
             idmap = dict(enumerate(ig.vs[self._node]))
             for e in ig.es:
                 t = e.tuple
-                yield dict({self._source: idmap[t[0]], self._destination: idmap[t[1]]},
-                            **e.attributes())
+                yield dict(
+                    {self._source: idmap[t[0]], self._destination: idmap[t[1]]},
+                    **e.attributes())
 
         self._check_mandatory_bindings(False)
         if self._node is None:
@@ -1065,9 +1098,9 @@ class Plotter(object):
 
         edata = get_edgelist(ig)
         ndata = [v.attributes() for v in ig.vs]
-        nodes = pandas.DataFrame(ndata, columns=ig.vs.attributes())
+        nodes = pd.DataFrame(ndata, columns=ig.vs.attributes())
         cols = [self._source, self._destination] + ig.es.attributes()
-        edges = pandas.DataFrame(edata, columns=cols)
+        edges = pd.DataFrame(edata, columns=cols)
         return (edges, nodes)
 
 
@@ -1089,6 +1122,7 @@ class Plotter(object):
         def get_nodelist(g):
             for n in g.nodes(data=True):
                 yield dict({self._node: n[0]}, **n[1])
+
         def get_edgelist(g):
             for e in g.edges(data=True):
                 yield dict({self._source: e[0], self._destination: e[1]}, **e[2])
@@ -1097,8 +1131,8 @@ class Plotter(object):
         self.networkx_checkoverlap(g)
         
         self._node = self._node or Plotter._defaultNodeId
-        nodes = pandas.DataFrame(get_nodelist(g))
-        edges = pandas.DataFrame(get_edgelist(g))
+        nodes = pd.DataFrame(get_nodelist(g))
+        edges = pd.DataFrame(get_edgelist(g))
         return (edges, nodes)
 
 
@@ -1117,18 +1151,18 @@ class Plotter(object):
                 error('%s attribute "%s" bound to "%s" does not exist.' % (typ, a, b))
 
 
-    def _plot_dispatch(self, graph, nodes, name, description, mode='json', metadata=None):
+    def _plot_dispatch(self, graph, nodes, name, description, mode='json', metadata=None, memoize=True):
 
-        if isinstance(graph, pandas.core.frame.DataFrame) \
-            or isinstance(graph, pa.Table) \
-            or ( not (maybe_cudf is None) and isinstance(graph, maybe_cudf.DataFrame) ):
-            return self._make_dataset(graph, nodes, name, description, mode, metadata)
+        if isinstance(graph, pd.core.frame.DataFrame) \
+                or isinstance(graph, pa.Table) \
+                or ( not (maybe_cudf is None) and isinstance(graph, maybe_cudf.DataFrame) ):
+            return self._make_dataset(graph, nodes, name, description, mode, metadata, memoize)
 
         try:
             import igraph
             if isinstance(graph, igraph.Graph):
                 (e, n) = self.igraph2pandas(graph)
-                return self._make_dataset(e, n, name, description, mode, metadata)
+                return self._make_dataset(e, n, name, description, mode, metadata. memoize)
         except ImportError:
             pass
 
@@ -1139,7 +1173,7 @@ class Plotter(object):
                isinstance(graph, networkx.classes.multigraph.MultiGraph) or \
                isinstance(graph, networkx.classes.multidigraph.MultiDiGraph):
                 (e, n) = self.networkx2pandas(graph)
-                return self._make_dataset(e, n, name, description, mode, metadata)
+                return self._make_dataset(e, n, name, description, mode, metadata, memoize)
         except ImportError:
             pass
 
@@ -1158,12 +1192,13 @@ class Plotter(object):
                      .dropna(subset=[self._source, self._destination])
 
         obj_df = elist.select_dtypes(include=[numpy.object_])
-        elist[obj_df.columns] = obj_df.apply(pandas.to_numeric, errors='ignore')
+        elist[obj_df.columns] = obj_df.apply(pd.to_numeric, errors='ignore')
 
         if nodes is None:
-            nodes = pandas.DataFrame()
-            nodes[nodeid] = pandas.concat([edges[self._source], edges[self._destination]],
-                                           ignore_index=True).drop_duplicates()
+            nodes = pd.DataFrame()
+            nodes[nodeid] = pd.concat(
+                [edges[self._source], edges[self._destination]],
+                ignore_index=True).drop_duplicates()
         else:
             self._check_bound_attribs(nodes, ['node'], 'Vertex')
 
@@ -1172,7 +1207,7 @@ class Plotter(object):
                      .drop_duplicates(subset=[nodeid])
 
         obj_df = nlist.select_dtypes(include=[numpy.object_])
-        nlist[obj_df.columns] = obj_df.apply(pandas.to_numeric, errors='ignore')
+        nlist[obj_df.columns] = obj_df.apply(pd.to_numeric, errors='ignore')
 
         return (elist, nlist)
 
@@ -1275,12 +1310,12 @@ class Plotter(object):
         }
         return (elist, nlist, encodings)
 
-    def _table_to_pandas(self, table) -> pandas.DataFrame:
+    def _table_to_pandas(self, table) -> pd.DataFrame:
 
         if table is None:
             return table
 
-        if isinstance(table, pandas.DataFrame):
+        if isinstance(table, pd.DataFrame):
             return table
 
         if isinstance(table, pa.Table):
@@ -1291,24 +1326,79 @@ class Plotter(object):
         
         raise Exception('Unknown type %s: Could not convert data to Pandas dataframe' % str(type(table)))
 
-    def _table_to_arrow(self, table) -> pa.Table:
+    def _table_to_arrow(self, table: any, memoize: bool = True) -> pa.Table:  # noqa: C901
+
+        logger.debug('_table_to_arrow of %s (memoize: %s)', type(table), memoize)
 
         if table is None:
             return table
 
         if isinstance(table, pa.Table):
+            #TODO: should we hash just in case there's an older-identity hash match?
             return table
         
-        if isinstance(table, pandas.DataFrame):
-            return pa.Table.from_pandas(table, preserve_index=False).replace_schema_metadata({})
+        if isinstance(table, pd.DataFrame):
+            hashed = None
+            if memoize:
+                
+                try:
+                    #https://stackoverflow.com/questions/31567401/get-the-same-hash-value-for-a-pandas-dataframe-each-time
+                    hashed = hashlib.sha256(pd.util.hash_pandas_object(table, index=True).values).hexdigest()
+                except TypeError:
+                    logger.warn('Failed memoization speedup attempt due to Pandas internal hash function failing. Continuing without memoization speedups.')
+
+                try:
+                    if hashed in Plotter._pd_hash_to_arrow:
+                        logger.debug('pd->arrow memoization hit: %s', hashed)
+                        return Plotter._pd_hash_to_arrow[hashed].v
+                    else:
+                        logger.debug('pd->arrow memoization miss for id (of %s): %s', len(Plotter._pd_hash_to_arrow), hashed)
+                except:
+                    logger.debug('Failed to hash pdf', exc_info=True)
+                    1
+
+            out = pa.Table.from_pandas(table, preserve_index=False).replace_schema_metadata({})
+
+            if memoize and (hashed is not None):
+                w = WeakValueWrapper(out)
+                cache_coercion(hashed, w)
+                Plotter._pd_hash_to_arrow[hashed] = w
+
+            return out
 
         if not (maybe_cudf is None) and isinstance(table, maybe_cudf.DataFrame):
-            return table.to_arrow()
+
+            hashed = None
+            if memoize:
+                #https://stackoverflow.com/questions/31567401/get-the-same-hash-value-for-a-pandas-dataframe-each-time
+                hashed = hashlib.sha256(table.hash_columns().tobytes()).hexdigest()
+                try:
+                    if hashed in Plotter._cudf_hash_to_arrow:
+                        logger.debug('cudf->arrow memoization hit: %s', hashed)
+                        return Plotter._cudf_hash_to_arrow[hashed].v
+                    else:
+                        logger.debug('cudf->arrow memoization miss for id (of %s): %s', len(Plotter._cudf_hash_to_arrow), hashed)
+                except:
+                    logger.debug('Failed to hash cudf', exc_info=True)
+                    1
+
+            out = table.to_arrow()
+
+            if memoize:
+                w = WeakValueWrapper(out)
+                cache_coercion(hashed, w)
+                Plotter._cudf_hash_to_arrow[hashed] = w
+
+            return out
         
         raise Exception('Unknown type %s: Could not convert data to Arrow' % str(type(table)))
 
 
-    def _make_dataset(self, edges, nodes, name, description, mode, metadata=None):
+    def _make_dataset(self, edges, nodes, name, description, mode, metadata=None, memoize: bool = True):  # noqa: C901
+
+        logger.debug('_make_dataset (mode %s, memoize %s) name:[%s] des:[%s] (e::%s, n::%s) ',
+            mode, memoize, name, description, type(edges), type(nodes))
+
         try:
             if len(edges) == 0:
                 warn('Graph has no edges, may have rendering issues')
@@ -1316,16 +1406,15 @@ class Plotter(object):
             1
 
         #compatibility checks
-        if (mode =='json') or (mode == 'vgraph'):
+        if (mode == 'json') or (mode == 'vgraph'):
             if not (metadata is None):
                 if ('bg' in metadata) or ('fg' in metadata) or ('logo' in metadata) or ('page' in metadata):
                     raise ValueError('Cannot set bg/fg/logo/page in api=1, api=2; try using api=3')
-            if not (self._complex_encodings is None \
-                or self._complex_encodings == {
+            if not (self._complex_encodings is None
+                or self._complex_encodings == {  # noqa: W503
                     'node_encodings': {'current': {}, 'default': {} },
-                    'edge_encodings': {'current': {}, 'default': {} }
-                }):
-                    raise ValueError('Cannot set complex encodings ".encode_[point/edge]_[feature]()" in api=1, api=2; try using api=3 or .bind()')
+                    'edge_encodings': {'current': {}, 'default': {} }}):
+                raise ValueError('Cannot set complex encodings ".encode_[point/edge]_[feature]()" in api=1, api=2; try using api=3 or .bind()')
 
         if mode == 'json':
             edges_df = self._table_to_pandas(edges)
@@ -1336,8 +1425,8 @@ class Plotter(object):
             nodes_df = self._table_to_pandas(nodes)
             return self._make_vgraph_dataset(edges_df, nodes_df, name)
         elif mode == 'arrow':
-            edges_arr = self._table_to_arrow(edges)
-            nodes_arr = self._table_to_arrow(nodes)
+            edges_arr = self._table_to_arrow(edges, memoize)
+            nodes_arr = self._table_to_arrow(nodes, memoize)
             return self._make_arrow_dataset(edges=edges_arr, nodes=nodes_arr, name=name, description=description, metadata=metadata)
             #token=None, dataset_id=None, url_params = None)
         else:
@@ -1350,7 +1439,7 @@ class Plotter(object):
         from .pygraphistry import PyGraphistry
 
         (elist, nlist) = self._bind_attributes_v1(edges, nodes)
-        edict = elist.where((pandas.notnull(elist)), None).to_dict(orient='records')
+        edict = elist.where((pd.notnull(elist)), None).to_dict(orient='records')
 
         bindings = {'idField': self._node or Plotter._defaultNodeId,
                     'destinationField': self._destination, 'sourceField': self._source}
@@ -1358,7 +1447,7 @@ class Plotter(object):
                    'bindings': bindings, 'type': 'edgelist', 'graph': edict}
 
         if nlist is not None:
-            ndict = nlist.where((pandas.notnull(nlist)), None).to_dict(orient='records')
+            ndict = nlist.where((pd.notnull(nlist)), None).to_dict(orient='records')
             dataset['labels'] = ndict
         return dataset
 
@@ -1375,9 +1464,9 @@ class Plotter(object):
         elist.drop([self._source, self._destination], axis=1, inplace=True)
 
         # Filter out nodes which have no edges
-        lnodes = pandas.concat([sources, dests], ignore_index=True).unique()
-        lnodes_df = pandas.DataFrame(lnodes, columns=[nodeid])
-        filtered_nlist = pandas.merge(lnodes_df, nlist, on=nodeid, how='left')
+        lnodes = pd.concat([sources, dests], ignore_index=True).unique()
+        lnodes_df = pd.DataFrame(lnodes, columns=[nodeid])
+        filtered_nlist = pd.merge(lnodes_df, nlist, on=nodeid, how='left')
 
         # Create a map from nodeId to a continuous range of integer [0, #nodes-1].
         # The vgraph protobuf format uses the continous integer ranger as internal nodeIds.
@@ -1391,7 +1480,7 @@ class Plotter(object):
 
         from .pygraphistry import PyGraphistry
 
-        au = ArrowUploader(
+        au : ArrowUploader = ArrowUploader(
             server_base_path=PyGraphistry.protocol() + '://' + PyGraphistry.server(),
             edges=edges, nodes=nodes,
             name=name, description=description,
@@ -1420,15 +1509,17 @@ class Plotter(object):
 
         res = copy.copy(self)
         driver = self._bolt_driver or PyGraphistry._config['bolt_driver']
+        if driver is None:
+            raise ValueError("BOLT connection information not provided. Must first call graphistry.register(bolt=...) or g.bolt(...).")
         with driver.session() as session:
             bolt_statement = session.run(query, **params)
             graph = bolt_statement.graph()
             edges = bolt_graph_to_edges_dataframe(graph)
             nodes = bolt_graph_to_nodes_dataframe(graph)
         return res\
-            .bind(\
-                node=node_id_key,\
-                source=start_node_id_key,\
+            .bind(
+                node=node_id_key,
+                source=start_node_id_key,
                 destination=end_node_id_key
             )\
             .nodes(nodes)\
@@ -1443,15 +1534,14 @@ class Plotter(object):
 
 
     def tigergraph(self,
-        protocol = 'http',
-        server = 'localhost',
-        web_port = 14240,
-        api_port = 9000,
-        db = None,
-        user = 'tigergraph',
-        pwd = 'tigergraph',
-        verbose = False
-    ):
+            protocol = 'http',
+            server = 'localhost',
+            web_port = 14240,
+            api_port = 9000,
+            db = None,
+            user = 'tigergraph',
+            pwd = 'tigergraph',
+            verbose = False):
         """Register Tigergraph connection setting defaults
     
         :param protocol: Protocol used to contact the database.
@@ -1545,26 +1635,26 @@ class Plotter(object):
                     import graphistry
                     tg = graphistry.tigergraph()
                     tg.gsql(\"\"\"
-INTERPRET QUERY () FOR GRAPH Storage { 
-    
-    OrAccum<BOOL> @@stop;
-    ListAccum<EDGE> @@edgeList;
-    SetAccum<vertex> @@set;
-    
-    @@set += to_vertex("61921", "Pool");
+                    INTERPRET QUERY () FOR GRAPH Storage { 
+                        
+                        OrAccum<BOOL> @@stop;
+                        ListAccum<EDGE> @@edgeList;
+                        SetAccum<vertex> @@set;
+                        
+                        @@set += to_vertex("61921", "Pool");
 
-    Start = @@set;
+                        Start = @@set;
 
-    while Start.size() > 0 and @@stop == false do
+                        while Start.size() > 0 and @@stop == false do
 
-      Start = select t from Start:s-(:e)-:t
-      where e.goUpper == TRUE
-      accum @@edgeList += e
-      having t.type != "Service";
-    end;
+                        Start = select t from Start:s-(:e)-:t
+                        where e.goUpper == TRUE
+                        accum @@edgeList += e
+                        having t.type != "Service";
+                        end;
 
-    print @@edgeList;
-  }
+                        print @@edgeList;
+                    }
                     \"\"\").plot()
 
        **Example: Full**
@@ -1573,33 +1663,26 @@ INTERPRET QUERY () FOR GRAPH Storage {
                     import graphistry
                     tg = graphistry.tigergraph()
                     tg.gsql(\"\"\"
-INTERPRET QUERY () FOR GRAPH Storage { 
-    
-    OrAccum<BOOL> @@stop;
-    ListAccum<EDGE> @@edgeList;
-    SetAccum<vertex> @@set;
-    
-    @@set += to_vertex("61921", "Pool");
+                    INTERPRET QUERY () FOR GRAPH Storage { 
+                        
+                        OrAccum<BOOL> @@stop;
+                        ListAccum<EDGE> @@edgeList;
+                        SetAccum<vertex> @@set;
+                        
+                        @@set += to_vertex("61921", "Pool");
 
-    Start = @@set;
+                        Start = @@set;
 
-    while Start.size() > 0 and @@stop == false do
+                        while Start.size() > 0 and @@stop == false do
 
-      Start = select t from Start:s-(:e)-:t
-      where e.goUpper == TRUE
-      accum @@edgeList += e
-      having t.type != "Service";
-    end;
+                        Start = select t from Start:s-(:e)-:t
+                        where e.goUpper == TRUE
+                        accum @@edgeList += e
+                        having t.type != "Service";
+                        end;
 
-    print @@my_edge_list;
-  }
+                        print @@my_edge_list;
+                    }
                     \"\"\", {'edges': 'my_edge_list'}).plot()
         """        
         return self._tigergraph.gsql(self, query, bindings, dry_run)
-
-
-
-
-
-
-
